@@ -1349,6 +1349,213 @@ Gold=Float, Green=Integer (Not Fixed), Blue=Integer (Fixed, Float Solution Type)
 
         self._add_figure(name=filename, figure=figure, title=figure_title)
 
+    def plot_heading_measurements(self):
+        """!
+        @brief Generate time series plots for heading (degrees) and baseline distance (meters) data.
+        """
+        if self.output_dir is None:
+            return
+
+        # Read the heading measurement data.
+        result = self.reader.read(message_types=[HeadingMeasurement], **self.params)
+        heading_data = result[HeadingMeasurement.MESSAGE_TYPE]
+
+        result = self.reader.read(message_types=[PoseMessage], **self.params)
+        primary_pose_data = result[PoseMessage.MESSAGE_TYPE]
+
+        if len(heading_data.p1_time) == 0:
+            self.logger.info('No heading measurement data available. Skipping plot.')
+            return
+
+        # Setup the figure.
+        fig = make_subplots(
+            rows=3, cols=1,
+            subplot_titles=(
+                'Heading, 2-sigma band',
+                'ENU/Baseline Distance',
+                'Solution Type'
+            ),
+            shared_xaxes=True,
+        )
+
+        fig.update_xaxes(title_text='Time (sec)', showticklabels=True)
+        fig.update_yaxes(title_text='Heading (deg)', rangemode='tozero', row=1, col=1)
+        fig.update_yaxes(title_text='Distance (m)', row=2, col=1)
+        fig.update_yaxes(
+            ticktext=['%s (%d)' % (e.name, e.value) for e in SolutionType],
+            tickvals=[e.value for e in SolutionType],
+            title_text='Solution Type',
+            row=3, col=1
+        )
+
+        fig.update_layout(title='Heading Plots', legend_traceorder='normal')
+
+        # First plot - heading in degrees with error bar
+        # calculate uncertainty envelope for heading
+        denom = heading_data.relative_position_enu_m[0]**2 + heading_data.relative_position_enu_m[1]**2
+        dh_e = heading_data.relative_position_enu_m[0] / denom
+        dh_n = heading_data.relative_position_enu_m[2] / denom
+
+        heading_std = np.sqrt(
+            (dh_e * heading_data.position_std_enu_m[0]) ** 2 +
+            (dh_n * heading_data.position_std_enu_m[1]) ** 2
+        )
+
+        envelope = np.arctan(
+            (2 * heading_std / heading_data.baseline_distance_m)
+        )
+        envelope *= 180. / np.pi
+
+        # Display the navigation engine's heading estimate, if available, for comparison with the heading sensor
+        # measurement.
+        if primary_pose_data is not None:
+            pose_heading_deg = 90.0 - primary_pose_data.ypr_deg[0]
+            pose_heading_deg[pose_heading_deg < 0.0] += 360.0
+            fig.add_trace(
+                go.Scatter(
+                    x=primary_pose_data.p1_time - float(self.t0),
+                    y=pose_heading_deg,
+                    customdata=primary_pose_data.p1_time,
+                    mode='lines',
+                    line={'color': 'yellow'},
+                    name='Primary Device Heading Estimate',
+                    hovertemplate='<b>Time</b>: %{x:.3f} sec (%{customdata:.3f} sec)'
+                                  '<br><b>Heading</b>: %{y:.2f} deg'
+                ),
+                row=1, col=1
+            )
+
+        heading_time = heading_data.p1_time - float(self.t0)
+
+        fig.add_trace(
+            go.Scatter(
+                x=heading_time,
+                y=heading_data.heading_true_north_deg,
+                customdata=heading_data.p1_time,
+                mode='markers',
+                marker={'size': 2},
+                name='Secondary Device Heading Measurement',
+                hovertemplate='<b>Time</b>: %{x:.3f} sec (%{customdata:.3f} sec)'
+                              '<br><b>Heading</b>: %{y:.2f} deg',
+                legendgroup='heading'
+            ),
+            row=1, col=1
+        )
+
+        idx = ~np.isnan(heading_data.heading_true_north_deg)
+
+        fig.add_trace(
+            go.Scatter(
+                x=heading_time[idx],
+                y=heading_data.heading_true_north_deg[idx] + envelope[idx],
+                mode='lines',
+                marker=dict(color="#444"),
+                line=dict(width=0),
+                legendgroup='heading',
+                showlegend=False,
+                hoverinfo='skip'
+            ),
+            row=1, col=1
+        )
+
+        fig.add_trace(
+            go.Scatter(
+                x=heading_time[idx],
+                y=heading_data.heading_true_north_deg[idx] - envelope[idx],
+                mode='lines',
+                marker=dict(color="#444"),
+                line=dict(width=0),
+                fillcolor='rgba(68, 68, 68, 0.3)',
+                fill='tonexty',
+                legendgroup='heading',
+                showlegend=False,
+                hoverinfo='skip'
+            ),
+            row=1, col=1
+        )
+
+        # Second plot - baseline, ENU components
+        fig.add_trace(
+            go.Scatter(
+                x=heading_time,
+                y=heading_data.relative_position_enu_m[0],
+                customdata=heading_data.p1_time,
+                hovertemplate='<b>Time</b>: %{x:.3f} sec (%{customdata:.3f} sec)'
+                              '<br><b>East</b>: %{y:.2f} m',
+                name='East'
+            ),
+            row=2, col=1
+        )
+
+        fig.add_trace(
+            go.Scatter(
+                x=heading_time,
+                y=heading_data.relative_position_enu_m[1],
+                customdata=heading_data.p1_time,
+                hovertemplate='<b>Time</b>: %{x:.3f} sec (%{customdata:.3f} sec)'
+                              '<br><b>North</b>: %{y:.2f} m',
+                name='North'
+            ),
+            row=2, col=1
+        )
+
+        fig.add_trace(
+            go.Scatter(
+                x=heading_time,
+                y=heading_data.relative_position_enu_m[2],
+                customdata=heading_data.p1_time,
+                hovertemplate='<b>Time</b>: %{x:.3f} sec (%{customdata:.3f} sec)'
+                              '<br><b>Up</b>: %{y:.2f} m',
+                name='Up'
+            ),
+            row=2, col=1
+        )
+
+        fig.add_trace(
+            go.Scatter(
+                x=heading_time,
+                y=heading_data.baseline_distance_m,
+                customdata=heading_data.p1_time,
+                hovertemplate='<b>Time</b>: %{x:.3f} sec (%{customdata:.3f} sec)'
+                              '<br><b>Baseline</b>: %{y:.2f} m',
+                name='Baseline'
+            ),
+            row=2, col=1
+        )
+
+        # 3rd plot - solution type
+        if primary_pose_data is not None:
+            fig.add_trace(
+                go.Scatter(
+                    x=primary_pose_data.p1_time - float(self.t0),
+                    y=primary_pose_data.solution_type,
+                    customdata=primary_pose_data.p1_time,
+                    mode='markers',
+                    marker={'color': 'yellow'},
+                    hovertemplate='<b>Time</b>: %{x:.3f} sec (%{customdata:.3f} sec)'
+                                  '<br><b>Solution</b>: %{text}',
+                    text=[str(SolutionType(s)) for s in primary_pose_data.solution_type],
+                    name='Primary Solution Type'
+                ),
+                row=3, col=1
+            )
+
+        fig.add_trace(
+            go.Scatter(
+                x=heading_time,
+                y=heading_data.solution_type,
+                customdata=heading_data.p1_time,
+                mode='markers',
+                hovertemplate='<b>Time</b>: %{x:.3f} sec (%{customdata:.3f} sec)'
+                              '<br><b>Solution</b>: %{text}',
+                text=[str(SolutionType(s)) for s in heading_data.solution_type],
+                name='Secondary Solution Type'
+            ),
+            row=3, col=1
+        )
+
+        self._add_figure(name='heading_measurement', figure=fig, title='Measurements: Heading')
+
     def plot_events(self):
         """!
         @brief Generate a table of event notifications.
@@ -1787,6 +1994,10 @@ Load and display information stored in a FusionEngine binary file.
         analyzer.plot_gnss_signal_status()
         analyzer.plot_gnss_skyplot()
 
+        # By default, we always plot heading measurements (i.e., output from a secondary heading device like an
+        # LG69T-AH), separate from other sensor measurements controlled by --measurements.
+        analyzer.plot_heading_measurements()
+
         if options.measurements:
             analyzer.plot_imu()
             analyzer.plot_wheel_data()
